<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class IFoodMenu
{
    private $iFoodMenuService;
    private $integrationsFactory;
    private $iFood;

    public function __construct(
        \App\Service\IFoodMenu $iFoodMenuService,
        \App\Factory\integrations $integrationsFactory,
        \App\Integration\IFood $iFood
        )
    {
        $this->iFoodMenuService = $iFoodMenuService;
        $this->integrationsFactory = $integrationsFactory;
        $this->iFood = $iFood;

    }

    function formatDate($originalDate)
    {
        $originalDate = date_create($originalDate);
        return date_format($originalDate, "d/m/Y H:i:s");
    }

    public function listMenus(Request $request, Response $response)
    {
        try {
            $menuList = [];
            $params = $request->getQueryParams();
            $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]) : "";
            $token = $this->iFood->getAuthToken();
            $data  = $this->iFoodMenuService->getMenuId($merchantId, $token);
            $info  = $this->iFood->getMerchantInfo($merchantId, $token);

            foreach ($data as $menu) {
                $menuList[] = [
                    "menuId" => $menu["catalogId"],
                    "menuStatus" => $menu["status"],
                    "lastModifiedAt" => $menu['modifiedAt'],
                ];
            }

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $menuList,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getMerchantInfo(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $merchantId = isset($params["idToken"]) ? strval($params["idToken"]): "";

            $info = $this->iFood->getMerchantInfo($merchantId);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $info,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getCategory(Request $request, Response $response)
    {
        try {
            $params = $request->getQueryParams();
            $menuId = isset($params["menuID"]) ? strval($params["menuID"]) : "";
            $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]) : "";

            $category = $this->iFoodMenuService->items($params);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $category,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }
    public function getItens(Request $request, Response $response)
    {
        try {
            $params   = $request->getParsedBody();
            $items = $this->iFoodMenuService->items($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $items,
            ]));

            return $response->withHeader('Content-Type', 'application/json');

        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getBranch(Request $request, Response $response)
    {
        try {
            $result = $this->iFoodMenuService->getBranch();


            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

   public function updateMenu(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $token = $this->iFood->getAuthToken();
            $merchantId = $params['merchantId'];
            $errors = array();

            foreach ($params['items'] as &$item) {
                switch($item['type']) {
                    case 'I':
                        if($item['initialSku'] != $item['finalSku']) {
                            $apiResponse = $this->iFoodMenuService->setItemExternalcode($merchantId, $item, $token);
                            if (!empty($apiResponse)) {
                                $item['errorDescription'] = $apiResponse;
                                $errors[] = $item;
                                $item['finalSku'] = null;
                            }
                        }

                        /*
                         * @TODO: Finalizar implementação para atualizar preço do item
                         */
                        // if ($item['initialPrice'] != $item['finalPrice']) {
                        //     $apiResponse = $this->iFoodMenuService->updateItemPrice($token, $merchantId, $item);
                        //     if (!empty($apiResponse)) {
                        //         $item['errorDescription'] = $apiResponse;
                        //         $errors[] = $item;
                        //         $item['finalPrice'] = null;
                        //     }
                        // }
                        break;
                    case 'C':
                        if($item['initialSku'] != $item['finalSku']) {
                            $apiResponse = $this->iFoodMenuService->setOptionExternalCode($merchantId, $item, $token);
                            if (!empty($apiResponse)) {
                                $item['errorDescription'] = $apiResponse;
                                $errors[] = $item;
                                $item['finalSku'] = null;
                            }

                            /*
                             * @TODO: Finalizar implementação para atualizar preço de complementos
                             */
                            // if ($item['initialPrice'] != $item['finalPrice']) {
                            //     $apiResponse = $this->iFoodMenuService->updateOptionPrice($token, $merchantId, $item);
                            //     if (!empty($apiResponse)) {
                            //         $item['errorDescription'] = $apiResponse;
                            //         $errors[] = $item;
                            //         $item['finalPrice'] = null;
                            //     }
                            // }
                        }
                        break;
                    default:
                        break;
                }
            }

            $this->integrationsFactory->logItemAlteration($params);

            if (!empty($errors)) {
                throw New \Exception("Houve um problema ao atualizar alguns itens.");
            }

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "Success",
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");

        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                    "relatedItems" => $errors,
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    private function formatShifts($openingHours) {
        $dayOfWeekMap = array(
            "MONDAY" => "Seg.",
            "TUESDAY" => "Ter.",
            "WEDNESDAY" => "Qua.",
            "THURSDAY" => "Qui.",
            "FRIDAY" => "Sex.",
            "SATURDAY" => "Sáb.",
            "SUNDAY" => "Dom."
        );

        $groupedShifts = array();
        foreach ($openingHours as $shift) {
            $day = $dayOfWeekMap[$shift['dayOfWeek']];
            if (!isset($groupedShifts[$day])) {
                $groupedShifts[$day] = array();
            }
            array_push($groupedShifts[$day], $shift);
        }

        $orderedDaysOfWeek = array("Dom.", "Seg.", "Ter.", "Qua.", "Qui.", "Sex.", "Sáb.");

        $formattedShifts = array();
        foreach ($orderedDaysOfWeek as $day) {
            if (!isset($groupedShifts[$day])) {
                array_push($formattedShifts, "<b>$day</b> Fechado");
            } else {
                $allDay = array_reduce($groupedShifts[$day], function ($carry, $item) {
                    return $carry && $item['duration'] === 1439;
                }, true);
                $closed = array_reduce($groupedShifts[$day], function ($carry, $item) {
                    return $carry && $item['duration'] === 0;
                }, true);
                if ($allDay) {
                    array_push($formattedShifts, "<b>$day</b> Aberto 24 horas");
                } elseif ($closed) {
                    array_push($formattedShifts, "<b>$day</b> Fechado");
                } else {
                    $times = array();
                    foreach ($groupedShifts[$day] as $shift) {
                        $start = date('H:i',strtotime($shift['start']));
                        $end = strtotime($start) + $shift['duration'] * 60;
                        $endStr = date('H:i', $end);
                        array_push($times, "$start ~ $endStr");
                    }
                    array_push($formattedShifts, "<b>$day</b> " . implode("; ", $times));
                }
            }
        }

        return $formattedShifts;
    }

    private function formatOperationStatus($status) {
        $response = array();
        foreach ($status as $operation) {
            $problems = array();
            if ($operation['state'] != 'OK') {
                foreach ($operation['validations'] as $validations) {
                    if ($validations['state'] != 'OK') {
                        $problems[] = [
                            "operation"   => $operation['operation'],
                            "state"       => $validations['state'],
                            "title"       => $validations['message']['title'],
                            "subtitle"    => $validations['message']['subtitle'],
                        ];
                    }
                }
            }
            $response[] = [
                "operation" => $operation['operation'],
                "state"     => $operation['state'],
                "problems"  => $problems,
            ];
        }
        return $response;
    }

    private function formatMerchantAvailabilityStatus($interruptions, $oppeningHours, $operationStatus) {

        $operationStatus = $this->formatOperationStatus($operationStatus);
        $formattedShifts = $this->formatShifts($oppeningHours['shifts']);

        $formattedInterruptions = [];
        $data = array();
        if (empty($interruptions)) {
           $formattedInterruptions = [];
        } else {
            foreach ($interruptions as $key => $value) {
                $formattedInterruptions[$key] = [
                    "id"            => $value['id'],
                    "description"   => $value['description'],
                    "start"         => $this->formatDate($value['start']),
                    "end"           => $this->formatDate($value['end'])
                ];
            }
        }

        $data = [
            "operations"    => $operationStatus,
            "interruptions" => $formattedInterruptions,
            "shifts" => $formattedShifts,
        ];
        return $data;
    }

    public function merchantAvailability(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $merchantId = $params['merchantId'];
            $token = $this->iFood->getAuthToken();
            $interruptions = $this->iFood->getMerchantInterruptions($merchantId, $token);
            $oppeningHours = $this->iFood->getOpeningHours($merchantId, $token);
            
            $merchantStatus = $this->iFood->getMerchantStatus($merchantId, $token);
            

            $data = $this->formatMerchantAvailabilityStatus($interruptions, $oppeningHours, $merchantStatus);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $data,
                ])
            );

            return $response->withHeader("Content-Type", "application/json");

        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function setItemExternalCode(Request $request, Response $response){
        try {
            $params = $request->getParsedBody();
            $token = $this->iFood->getAuthToken();
            $result = $this->iFoodMenuService->setItemExternalcode($token, $params);
            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }

        return $result;
    }
}
