<?php
namespace App\Factory;

use Doctrine\Common\EventManager;
use Doctrine\DBAL\Configuration as DoctrineConfiguration;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Event\Listeners\OracleSessionInit;
use Doctrine\DBAL\DriverManager;

class Operator {
    private $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function getSaaSConnection() {
      $evm = new EventManager();
      $connectionParams = [
        'driver'      => $_ENV['DB_SAAS_DRIVER'],
        'host'        => $_ENV['DB_SAAS_HOST'],
        'dbname'      => $_ENV['DB_SAAS_NAME'],
        'user'        => $_ENV['DB_SAAS_USER'],
        'password'    => $_ENV['DB_SAAS_PASSWORD'],
        'charset'     => $_ENV['DB_SAAS_CHARSET'],
        'service'     => true,
        'pooled'      => filter_var($_ENV['DB_SAAS_POOLED'], FILTER_VALIDATE_BOOLEAN),
        'sessionMode' => ($_ENV['DB_SAAS_SESSION_MODE'] === "OCI_CRED_EXT") ? OCI_CRED_EXT : OCI_NO_AUTO_COMMIT
      ];
      if (empty($connectionParams['host'])) {
        unset($connectionParams['host']);
      }

      $config = new DoctrineConfiguration();
      $evm->addEventSubscriber(new OracleSessionInit());
      $connectionParams['platform'] = new \Doctrine\DBAL\Platforms\OraclePlatform();

      return DriverManager::getConnection($connectionParams, $config, $evm);
    }
    
    public function getOperatorData($emailOrNmAcesso, $password, $connection = null) {
      $isSaaS = $_ENV['IS_SAAS'] === 'YES';
      $tableName = $isSaaS ? 'USR_ORG_SAAS.OPERADOR' : 'OPERADOR';
      $connection = $connection ? $connection : $this->connection;
      $sql = "
        SELECT
          CDOPERADOR  AS ID,
          NMOPERADOR  AS NAME,
          NRORG       AS ORGANIZATION_ID,
          DSEMAILOPER AS EMAIL,
          IDIDIOMA,
          NMACESSO,
          NMCOMPLOPER,
          CDEMPRESA,
          NRPARCNEGOCIO,
          NRORG,
          NRORGTRAB,
          NRESTRUTTRAB,
          CDFILIALSELECIONADA,
          NRORGOPER,
          CASE WHEN IDOPERATIVO = 'A'
            THEN 'Y'
            ELSE 'N'
          END AS IS_ACTIVE
        FROM $tableName
        WHERE
          (UPPER(NMACESSO) = UPPER(:OPERADOR) OR DSEMAILOPER = :OPERADOR)
          AND CDSENHAOPERWEB = :PASSWORD
      ";

      $stmt = $connection->prepare($sql);
      $stmt->bindValue("OPERADOR", $emailOrNmAcesso);
      $stmt->bindValue("PASSWORD", md5($password));
      $result = $stmt->executeQuery();
      return $result->fetchAssociative();
    }

    public function getSMTP() {
      $sql = "
        SELECT DSSMTPAUTVND, DSEMAILAUVND, NRPORTAAUTVND, CDSENHAAUTVNDWEB,
          DSEMAILCLUSO, LOWER(DSSMTPCRIPT) AS DSSMTPCRIPT, IDTPAUTHSMTP
        FROM PARAMGERAL
      ";
  
      $stmt = $this->connection->prepare($sql);
      $result = $stmt->executeQuery();
      return $result->fetchAllAssociative();
    }

    public function updatePassword($encryptedNewPassword, $encryptedNewPasswordDelphi, $operatorId) {  
      $sql = "
        UPDATE OPERADOR
          SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
              CDSENHOPER     = :CDSENHOPER,
              DTULTATU       = TO_DATE(TO_CHAR(SYSDATE, 'DD/MM/YYYY HH24:MI:SS'), 'DD/MM/YYYY HH24:MI:SS'),
              IDSTATUSENHA   = 'A'
        WHERE CDOPERADOR     = :CDOPERADOR
      ";
  
      $stmt = $this->connection->prepare($sql);
      $stmt->bindValue("CDSENHAOPERWEB", $encryptedNewPassword);
      $stmt->bindValue("CDSENHOPER", $encryptedNewPasswordDelphi);
      $stmt->bindValue("CDOPERADOR", $operatorId);
      $result = $stmt->executeQuery();
      return $result;
    }
    
    public function getPassword($operatorId) {
      $sql = "
        SELECT CDSENHAOPERWEB
          FROM OPERADOR
        WHERE  CDOPERADOR = :CDOPERADOR
      ";

      $stmt = $this->connection->prepare($sql);
      $stmt->bindValue("CDOPERADOR", $operatorId);
      $result = $stmt->executeQuery();
      return $result->fetchAllAssociative();
    }

	  public function validateEmail($email) {
		  $sql = "
          SELECT CDOPERADOR AS ID, NMOPERADOR AS NAME, DSEMAILOPER AS EMAIL
          FROM OPERADOR 
          WHERE DSEMAILOPER = :EMAIL
		    ";

		$stmt = $this->connection->prepare($sql);
		$stmt->bindValue("EMAIL", $email);
		$result = $stmt->executeQuery();
    return $result->fetchAllAssociative();
	  }

    public function resetPassword($password, $name) {
        $sql = "
            UPDATE OPERADOR
            SET CDSENHAOPERWEB = :CDSENHAOPERWEB
            WHERE UPPER(NMACESSO) = UPPER(:NMACESSO) 
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDSENHAOPERWEB", $password);
        $stmt->bindValue("NMACESSO", $name);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }

    public function login($operator, $password) {
        $sql = "
        SELECT CDOPERADOR AS ID, NMOPERADOR AS NAME, NRORG AS ORGANIZATION_ID, DSEMAILOPER AS EMAIL,
            IDIDIOMA AS IDIDIOMA, NMACESSO AS NMACESSO, NMCOMPLOPER AS NMCOMPLOPER, CDEMPRESA AS CDEMPRESA,
            NRPARCNEGOCIO AS NRPARCNEGOCIO, NRORG AS NRORG, NRORGTRAB AS NRORGTRAB, NRESTRUTTRAB AS NRESTRUTTRAB,
            CDFILIALSELECIONADA AS CDFILIALSELECIONADA, NRORGOPER AS NRORGOPER,
            CASE WHEN IDOPERATIVO = 'A' THEN 'Y' ELSE 'N' END AS IS_ACTIVE 
        FROM OPERADOR 
        WHERE (UPPER(NMACESSO) = UPPER(:OPERATOR) OR DSEMAILOPER = :OPERATOR) AND CDSENHAOPERWEB = :CDSENHAOPERWEB
        ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("OPERATOR", $operator);
        $stmt->bindValue("CDSENHAOPERWEB", $password);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }
    
    public function getAccessProfileByOperator($params)	{
  		$sql = "
  		SELECT
        MAX( DECODE( NVL(GP.IDDASHBOARD,      'N'), 'N', 'N', 'S')) AS IDDASHBOARD,
        MAX( DECODE( NVL(GP.IDTEMPOATRASO,    'N'), 'N', 'N', 'S')) AS IDTEMPOATRASO,
        MAX( DECODE( NVL(GP.IDBLOQPROD,       'N'), 'N', 'N', 'S')) AS IDBLOQPROD,
        MAX( DECODE( NVL(GP.IDBLOQLOJA,       'N'), 'N', 'N', 'S')) AS IDBLOQLOJA,
        MAX( DECODE( NVL(GP.IDCANCELAPEDIDOS, 'N'), 'N', 'N', 'S')) AS IDCANCELAPEDIDOS,
        MAX( DECODE( NVL(GP.IDADIANTAPEDIDOS, 'N'), 'N', 'N', 'S')) AS IDADIANTAPEDIDOS,
        MAX( DECODE( NVL(GP.IDREMOVELOGERRO,  'N'), 'N', 'N', 'S')) AS IDREMOVELOGERRO
      FROM
        OPERGRUPOP OPGR INNER JOIN
        GRUPOPER   GP 
          ON GP.CDGRUPOPER = OPGR.CDGRUPOPER
      WHERE OPGR.CDOPERADOR = :CDOPERADOR";
  
  		$stmt = $this->connection->prepare($sql);
  		$stmt->bindValue("CDOPERADOR", $params['CDOPERADOR']);
  
  		$result = $stmt->executeQuery();

    	return $result->fetchAssociative();
	  }
}