<?php

namespace App\Integration;

class ApiRequest {

    public function send($url, $options = []) {
        $curl = curl_init();
        $method = $options['method'] ?? 'GET';
        $supressError = isset($options['supressError']) && $options['supressError'] === true;
        $curlOptions = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $method,
        ];
        
        if (isset($options['header'])) {
            $curlOptions[CURLOPT_HTTPHEADER] = $options['header'];
        }
        
        if (isset($options['data']) && $method !== 'GET') {
            $curlOptions[CURLOPT_POSTFIELDS] = $options['data'];
        }
        if (isset($_ENV['INTEGRATION_USE_SSL']) && $_ENV['INTEGRATION_USE_SSL'] === 'NO') {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);    
        } 

        curl_setopt_array($curl, $curlOptions);

        $curlResponse = curl_exec($curl);
        $errorMessage = curl_errno($curl);
        curl_close($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($errorMessage) {
            if ($supressError) {
                return [
                    'response' => $curlResponse,
                    'httpCode' => $httpCode,
                    'message'  => $errorMessage,
                ];
            }
            throw new \Exception($errorMessage);
        }
        if (!in_array($httpCode, [200, 201, 202, 204])) {
            if ($supressError) {
                return [
                    'response' => $curlResponse,
                    'httpCode' => $httpCode,
                    'message'  => $errorMessage,
                ];
            }
            throw new \Exception("Request fail with http_code: $httpCode");
        }
        return $curlResponse;
    }
    
    public function get($url, $options = []) {
        $queryParams = '';
        if (isset($options['data']) && !empty($options['data'])) {
            $queryParams = '?' . http_build_query($options['data']);
        }
        return $this->send($url . $queryParams, $options);
    }

    public function patch($url, $options) {
        return $this->send($url, array_merge($options, ['method' => 'PATCH']));
    }

    public function post($url, $options) {
        return $this->send($url, array_merge($options, ['method' => 'POST']));
    }

    public function put($url, $options) {
        return $this->send($url, array_merge($options, ['method' => 'PUT']));
    }

    public function delete($url, $options) {
        return $this->send($url, array_merge($options, ['method' => 'DELETE']));
    }
}