<?php
namespace App\Service;

use App\Exception;
class Operator {

    private $util;
    private $operatorFactory;
	private $smtp;

    public function __construct (\App\Util\Util $util,
                                \App\Factory\Operator $operatorFactory,
                                \App\Service\SMTPService $smtp
                                ) {

        $this->util             = $util;
        $this->operatorFactory  = $operatorFactory;
        $this->smtp             = $smtp;
    }

    private function callLicenseLogin($email, $password) {
        try {
            $endpoint = $this->util->getEnv("LOGIN_ENDPOINT");
    
            $requestData = [
                "requestType" => "FilterData",
                "filter" => [
                    [
                        "name" => "LOGIN",
                        "operator" => "=",
                        "value" => $email
                    ],
                    [
                        "name" => "PASSWORD",
                        "operator" => "=",
                        "value" => $password
                    ]
                ]
            ];
            $ch = curl_init($endpoint);
            
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($requestData));
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            
            $response = curl_exec($ch);
            $http = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            if ($http !== 200) {
                throw new \Exception('Falha para buscar as informações no endpoint de login');
            }

            $result = json_decode($response, true);

            return $result;
        } catch(\Exception $e) {
            throw new \Exception ($e->getMessage());
        }
    }

    private function loginDBSaaS($email, $password) {
        try {
            $conn = $this->operatorFactory->getSaaSConnection();

            $operator = $this->operatorFactory->getOperatorData($email, $password, $conn);
            return [
                'error' => null,
                'operator' => $operator,
            ];
        } catch(\Exception $err) {
            return [
                'error' => $err,
                'operator' => null,
            ];
        }
    }

    public function operatorLogin($email, $password) {
        try {
            $isSaaS = $this->util->getEnv("IS_SAAS") === 'YES';
            $operator = null;
            if ($isSaaS) {
                $operatorData = $this->loginDBSaaS($email, $password);

                if (isset($operatorData['operator']) && $operatorData['operator']) {
                    $operator = $operatorData['operator'];
                } else {
                    $operatorData = $this->callLicenseLogin($email, $password);
                    if (isset($operatorData['dataset']['operador'])) {
                        $operator = $operatorData['dataset']['operador'];
                    }
                }
            } else {
                $result = $this->operatorFactory->getOperatorData($email, $password);
                if (is_array($result) && !empty($result)) {
                    $operator = $result[0];
                }
            }

            if ($operator === null || empty($operator)) {
                throw new \Exception('Usuário/Senha inválido');
            }

            if($operator['IS_ACTIVE'] === 'N') {
                throw new \Exception('Operador inativo');
            }

            return $operator;
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage());
        }
    }

    public function updatePassword($currentPassword, $newPassword, $operatorEmail, $operatorName, $operatorId) { 
        $currentPassword = $this->util->encrypt($currentPassword);
        $toCheckPassword = $this->operatorFactory->getPassword($operatorId);

        if ($currentPassword !== $toCheckPassword[0]['CDSENHAOPERWEB']) {
            return array(
                'error' => true,
				'message' => 'Senha atual inválida.'
			);
		}

        $encryptedNewPassword       = $this->util->encrypt($newPassword);
        $encryptedNewPasswordDelphi = $this->util->getDelphiPasswordEncrypted($newPassword, true);

        if($encryptedNewPassword === $toCheckPassword[0]['CDSENHAOPERWEB']) {
            return array(
                'error' => true,
				'message' => 'Digite uma senha diferente da atual.'
			);
        }

        $updatePassword = $this->operatorFactory->updatePassword($encryptedNewPassword, 
             $encryptedNewPasswordDelphi, $operatorId);

        $rememberEmail = $this->rememberPasswordEmail($operatorEmail, $operatorName);

        if ($updatePassword->rowCount() > 0) {
            return array(
			    'error'   => false,
			    'message' => 'Sua senha foi alterada com successo.',
                'sendEmail'   => $rememberEmail
		    );
        } else {
           return array(
                'error'   => true,
                'message' => 'Desculpe, ocorreu um erro ao tentar atualizar a senha. Por favor, verifique os dados inseridos e tente novamente.',
                'sendEmail'   => $rememberEmail
           ); 
        }
    }

    public function rememberPasswordEmail($operatorEmail, $operatorName) {
        $subject = 'Alteração de Senha';
        $content = file_get_contents(__DIR__ . "/../../templates/rememberPasswordEmailTemplate.html");
        return $this->smtp->sendEmail($operatorName, $operatorEmail, $subject, $content);
    }

    public function resetPasswordEmail($DSEMAILOPER, $operatorName, $token) {
        $subject = 'Redefinição de Senha';
        $content = file_get_contents(__DIR__ . "/../../templates/resetPasswordEmailTemplate.html");
        $content = str_replace(['{token}', '{controlURL}'], [$token, $this->util->getenv("CONTROL_URL")], $content);
        return $this->smtp->sendEmail($operatorName, $DSEMAILOPER, $subject, $content);
    }

    public function validateEmail ($email) {
        $dataOperator = $this->operatorFactory->validateEmail($email);
        if(empty($dataOperator)) {
            throw new \Exception("Email não encontrado.");
        }

        return $dataOperator;
    }
    
    public function getAccessProfileByOperator($params) {
        return $this->operatorFactory->getAccessProfileByOperator($params);
    }

    public function resetPassword ($password, $id, $DSEMAILOPER, $operatorName) {
        $delphiPassword = $this->util->getDelphiPasswordEncrypted($password);
        $encryptedPassword = $this->util->encrypt($password);
        $updatePassword = $this->operatorFactory->updatePassword($encryptedPassword, $delphiPassword, $id);

        $rememberEmail = $this->rememberPasswordEmail($DSEMAILOPER, $operatorName);

        if ($updatePassword->rowCount() > 0) {
            return array(
			    'error'   => false,
			    'message' => 'Sua senha foi alterada com successo.',
                'sendEmail'   => $rememberEmail
		    );
        }
    }
}
