<?php

namespace App\Service;

class RappiMenu {
    
    private $rappi;
    private $integrationFactory;

    public function __construct(\App\Integration\Rappi $rappi, \App\Factory\Integrations $integrationFactory) {
        $this->rappi = $rappi;
        $this->integrationFactory = $integrationFactory;
    }
    
    function updateMenu($token, $params) {
        $menuStatus = $this->rappi->getMenuStatus($token, $params['merchantId']);
        if($menuStatus['status'] != 'Disponível') {
            throw new \Exception("O cardápio não está disponível para alterações");
        }

        $currentMenu = $this->rappi->getMenu($token,$params['merchantId']);
        foreach ($params['items'] as $modifiedItem) {
            if ($modifiedItem['type'] === 'C') {
                $parentIndex = $modifiedItem['parentIndex'];
                $arrayIndex = $modifiedItem['arrayIndex'];

                $currentMenu['items'][$parentIndex]['children'][$arrayIndex]['sku'] = $modifiedItem['finalSku'];
                $currentMenu['items'][$parentIndex]['children'][$arrayIndex]['price'] = $modifiedItem['finalPrice'];
            } else {
                $arrayIndex = $modifiedItem['arrayIndex'];

                $currentMenu['items'][$arrayIndex]['sku'] = $modifiedItem['finalSku'];
                $currentMenu['items'][$arrayIndex]['price'] = $modifiedItem['finalPrice'];
            }
        }
        $response = $this->rappi->postMenu($token, $currentMenu);

        return $response;
    }

    private function itemValidations(&$item) {
        $item['hasInvalidOptions'] = false;

        if ($item['combo'] && !empty($item['children'])) {
            $isValidItem = strlen($item['sku']) === 10;

            if (!$isValidItem) {
                $item['hasInvalidOptions'] = true;
                return;
            } else {
                foreach ($item['children'] as &$option) {
                    $isValidOption = $option['price'] > 0 && strlen($option['sku']) === 10;
                    if (!$isValidOption) {
                        $item['hasInvalidOptions'] = true;
                        return;
                    }
                }
            }

        } elseif (!$item['combo'] && !empty($item['children'])) {
            $isValidItem = $item['price'] >= 0.01 && strlen($item['sku']) === 10;

            if (!$isValidItem) {
                $item['hasInvalidOptions'] = true;
                return;
            } else {
                foreach ($item['children'] as $option) {
                    $isValidOption = (empty($option['price']) && strstr($option['sku'], "|")) ||
                                    ($option['price'] >= 0.01 && (strstr($option['sku'], "|") || strlen($option['sku']) === 10 ));

                    if (!$isValidOption) {
                        $item['hasInvalidOptions'] = true;
                        return;
                    }
                }
            }
        } else {
            $isValidItem = strlen($item['sku']) === 10 && $item['price'] >= 0.01;
            if (!$isValidItem) {
                $item['hasInvalidOptions'] = true;
            }
        }
    }

    function formatData(&$apiResponse) {
        $formattedData = [];
        $count = 0;

        foreach ($apiResponse['items'] as $index => $item) {
            $this->itemValidations($item);
            $categoryId = $item['category']['id'];
            $categoryName = $item['category']['name'];

            if (!isset($formattedData[$categoryId])) {
                $formattedData[$categoryId] = [
                    'id' => $categoryId,
                    'name' => $categoryName,
                    'items' => [],
                ];
            }

            $isValid = !$item['hasInvalidOptions'];

            $formattedData[$categoryId]['items'][] = [
                'id'            => $item['rappiIds'][0],
                'sku'  => $item['sku'],
                'name'          => $item['name'],
                'price'         => $item['price'],
                'combo'         => $item['combo'],
                'isValid'       => $isValid,
                "type"          => "I",
                "arrayIndex"    => $index,
            ];

            $lastParentId = $item['rappiIds'][0];

            if (!empty($item['children'])) {
                $optionGroup = [];

                foreach ($item['children'] as $optionIndex => $option) {
                    if (isset($option['category']) && !empty($option['category']) && !in_array($option['category']['id'], $optionGroup)) {
                        $formattedData[$categoryId]['items'][] = [
                            'parent_id' => $lastParentId,
                            'id'        => $option['category']['id'],
                            'name'      => $option['category']['name'],
                            'isValid'   => $isValid,
                        ];

                        $optionGroup[] = $option['category']['id'];
                    }

                    $formattedData[$categoryId]['items'][] = [
                        'parent_id'     => $option['category']['id'] ?? $lastParentId,
                        'id'            => $option['rappiIds'][0] ?? null,
                        'sku'           => $option['sku'],
                        'name'          => $option['name'],
                        'price'         => $option['price'],
                        'isValid'       => $isValid,
                        "type"          => "C",
                        "arrayIndex"    => $optionIndex,
                        "parentIndex"   => $index,
                    ];
                }
            }
        }

        return array_values($formattedData);
    }
}
